@@ -108,8 +108,8 @@ ole_get_block (GsfInfileMSOle const *ole, guint32 block, guint8 *buffer)
 	g_return_val_if_fail (block < ole->info->max_block, NULL);
 
 	if (gsf_input_seek (ole->input,
-		(off_t)(OLE_HEADER_SIZE + (block << ole->info->bb.shift)),
-		GSF_SEEK_SET) < 0)
+		(gsf_off_t)(OLE_HEADER_SIZE + (block << ole->info->bb.shift)),
+		G_SEEK_SET) < 0)
 		return NULL;
 
 	return gsf_input_read (ole->input, ole->info->bb.size, buffer);
@@ -436,7 +436,7 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	guint32 metabat_block, *ptr;
 
 	/* check the header */
-	if (gsf_input_seek (ole->input, 0, GSF_SEEK_SET) ||
+	if (gsf_input_seek (ole->input, (gsf_off_t) 0, G_SEEK_SET) ||
 	    NULL == (header = gsf_input_read (ole->input, OLE_HEADER_SIZE, NULL)) ||
 	    0 != memcmp (header, signature, sizeof (signature))) {
 		if (err != NULL)
@@ -480,6 +480,10 @@ ole_init_info (GsfInfileMSOle *ole, GError **err)
 	info->max_block	     = (gsf_input_size (ole->input) - OLE_HEADER_SIZE) / info->bb.size;
 	info->sb_file	     = NULL;
 
+	if (info->num_sbat == 0 && info->sbat_start != BAT_MAGIC_END_OF_CHAIN) {
+		g_warning ("There is are not supposed to be any blocks in the small block allocation table, yet there is a link to some.  Ignoring it.");
+	}
+
 	/* very rough heuristic, just in case */
 	if (num_bat < info->max_block) {
 		info->bb.bat.num_blocks = num_bat * (info->bb.size / BAT_INDEX_SIZE);
@@ -633,8 +637,8 @@ gsf_infile_msole_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 		/* optimization don't see if we don't need to */
 		if (ole->cur_block != first_block) {
 			if (gsf_input_seek (ole->input,
-				(off_t)(OLE_HEADER_SIZE + (ole->bat.block [first_block] << ole->info->bb.shift) + offset),
-				GSF_SEEK_SET) < 0)
+				(gsf_off_t)(OLE_HEADER_SIZE + (ole->bat.block [first_block] << ole->info->bb.shift) + offset),
+				G_SEEK_SET) < 0)
 				return NULL;
 		}
 		ole->cur_block = last_block;
@@ -671,7 +675,7 @@ gsf_infile_msole_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 }
 
 static gboolean
-gsf_infile_msole_seek (GsfInput *input, off_t offset, GsfOff_t whence)
+gsf_infile_msole_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 {
 	GsfInfileMSOle *ole = GSF_INFILE_MSOLE (input);
 
@@ -693,7 +697,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent, MSOleDirent *dirent)
 
 	child = ole_dup (parent);
 	child->dirent = dirent;
-	gsf_input_set_size (GSF_INPUT (child), dirent->size);
+	gsf_input_set_size (GSF_INPUT (child), (gsf_off_t) dirent->size);
 
 	/* The root dirent defines the small block file */
 	if (dirent->index != 0) {
@@ -730,7 +734,7 @@ gsf_infile_msole_new_child (GsfInfileMSOle *parent, MSOleDirent *dirent)
 
 		for (i = 0 ; i < child->bat.num_blocks; i++)
 			if (gsf_input_seek (GSF_INPUT (sb_file),
-				(off_t)(child->bat.block [i] << info->sb.shift), GSF_SEEK_SET) < 0 ||
+				(gsf_off_t)(child->bat.block [i] << info->sb.shift), G_SEEK_SET) < 0 ||
 			    (data = gsf_input_read (GSF_INPUT (sb_file),
 				info->sb.size, 
 				child->stream.buf + (i << info->sb.shift))) == NULL) {
@@ -849,7 +853,7 @@ gsf_infile_msole_new (GsfInput *source, GError **err)
 	ole = g_object_new (GSF_INFILE_MSOLE_TYPE, NULL);
 	g_object_ref (G_OBJECT (source));
 	ole->input = source;
-	gsf_input_set_size (GSF_INPUT (ole), 0);
+	gsf_input_set_size (GSF_INPUT (ole), (gsf_off_t) 0);
 
 	if (ole_init_info (ole, err)) {
 		g_object_unref (G_OBJECT (ole));
