@@ -76,6 +76,9 @@ struct _GsfInfileZip {
 	GList     *dirent_list;
 
 	ZipDirent *dirent;
+
+	guint8   *buf;
+	size_t    buf_size;
 };
 
 typedef struct {
@@ -85,13 +88,13 @@ typedef struct {
 #define GSF_INFILE_ZIP_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), GSF_INFILE_ZIP_TYPE, GsfInfileZipClass))
 #define GSF_IS_INFILE_ZIP_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GSF_INFILE_ZIP_TYPE))
 
-static off_t
+static gsf_off_t
 zip_find_trailer (GsfInfileZip *zip)
 {
 	static guint8 const trailer_signature[] =
 		{ 'P', 'K', 0x05, 0x06 };
-	off_t offset, trailer_offset;
-	size_t filesize, maplen;
+	gsf_off_t offset, trailer_offset, filesize;
+	size_t maplen;
 	guint8 const *data;
 
 	filesize = gsf_input_size (zip->input);
@@ -99,7 +102,12 @@ zip_find_trailer (GsfInfileZip *zip)
 		return -1;
 
 	trailer_offset = filesize;
-	maplen = filesize & (ZIP_BUF_SIZE - 1);
+	maplen = filesize;
+	if ((gsf_off_t) maplen != filesize) { /* Check for overflow */
+		g_warning ("File too large");
+		return -1;
+	}
+	maplen &= (ZIP_BUF_SIZE - 1);
 	if (maplen == 0)
 		maplen = ZIP_BUF_SIZE;
 	offset = filesize - maplen; /* offset is now BUFSIZ aligned */
@@ -107,7 +115,7 @@ zip_find_trailer (GsfInfileZip *zip)
 	while (TRUE) {
 		guchar *p, *s;
 
-		if ((gsf_input_seek (zip->input, offset, GSF_SEEK_SET)) < 0)
+		if ((gsf_input_seek (zip->input, offset, G_SEEK_SET)) < 0)
 			return -1;
 
 		if ((data = gsf_input_read (zip->input, maplen, NULL)) == NULL)
@@ -135,7 +143,8 @@ zip_find_trailer (GsfInfileZip *zip)
 
 		/* outer loop step */
 		offset -= ZIP_BUF_SIZE / 2;
-		maplen = ZIP_BUF_SIZE;
+		maplen = MIN (filesize - offset, ZIP_BUF_SIZE);
+		trailer_offset = offset + maplen;
 
 		if (filesize - offset > 64 * 1024)
 			return -1;
@@ -145,7 +154,7 @@ zip_find_trailer (GsfInfileZip *zip)
 }
 
 static ZipDirent *
-zip_dirent_new (GsfInfileZip *zip, off_t *offset)
+zip_dirent_new (GsfInfileZip *zip, gsf_off_t *offset)
 {
 	static guint8 const dirent_signature[] =
 		{ 'P', 'K', 0x01, 0x02 };
@@ -156,7 +165,7 @@ zip_dirent_new (GsfInfileZip *zip, off_t *offset)
 	gchar *name;
 
 	/* Read data and check the header */
-	if (gsf_input_seek (zip->input, *offset, GSF_SEEK_SET) ||
+	if (gsf_input_seek (zip->input, *offset, G_SEEK_SET) ||
 	    NULL == (data = gsf_input_read (zip->input, ZIP_DIRENT_SIZE, NULL)) ||
 	    0 != memcmp (data, dirent_signature, sizeof (dirent_signature))) {
 		return NULL;
@@ -268,25 +277,32 @@ zip_init_info (GsfInfileZip *zip, GError **err)
 	guint16 entries, i;
 	guint32 dir_pos;
 	ZipInfo *info;
-	off_t offset;
+	gsf_off_t offset;
 
 	/* Check the header */
-	if (gsf_input_seek (zip->input, 0, GSF_SEEK_SET) ||
+	if (gsf_input_seek (zip->input, (gsf_off_t) 0, G_SEEK_SET) ||
 	    NULL == (header = gsf_input_read (zip->input, ZIP_HEADER_SIZE, NULL)) ||
 	    0 != memcmp (header, header_signature, sizeof (header_signature))) {
-		g_set_error (err, gsf_input_error (), 0, "No Zip signature");
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+				"No Zip signature");
+		return TRUE;
 	}
 
 	/* Find and check the trailing header */
 	offset = zip_find_trailer (zip);
 	if (offset < 0) {
-		g_set_error (err, gsf_input_error (), 0, "No Zip trailer");
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+			     "No Zip trailer");
 		return TRUE;
 	}
 
-	if (gsf_input_seek (zip->input, offset, GSF_SEEK_SET) ||
+	if (gsf_input_seek (zip->input, offset, G_SEEK_SET) ||
 	    NULL == (trailer = gsf_input_read (zip->input, ZIP_TRAILER_SIZE, NULL))) {
-		g_set_error (err, gsf_input_error (), 0, "Error reading Zip signature");
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+					    "Error reading Zip signature");
 		return TRUE;
 	}
 
@@ -306,7 +322,10 @@ zip_init_info (GsfInfileZip *zip, GError **err)
 
 		d = zip_dirent_new (zip, &offset);
 		if (d == NULL) {
-			g_set_error (err, gsf_input_error (), 0, "Error reading zip dirent");
+			if (err != NULL)
+				*err = g_error_new
+					(gsf_input_error (), 0,
+					 "Error reading zip dirent");
 			return TRUE;
 		}
 
@@ -325,8 +344,9 @@ gsf_infile_zip_dup (GsfInput *src_input, GError **err)
 	GsfInfileZip *dst = zip_dup (src);
 
 	if (dst == NULL) {
-		g_set_error (err, gsf_input_error (), 0, "Something went wrong in zip_dup.");
-		return NULL;
+		if (err != NULL)
+			*err = g_error_new (gsf_input_error (), 0,
+					    "Something went wrong in zip_dup.");		return NULL;
 	}
 
 	dst->dirent = src->dirent;
@@ -348,7 +368,7 @@ zip_update_stream_in (GsfInfileZip const *zip)
 
 	read_now = (dirent->crestlen > ZIP_BLOCK_SIZE) ? ZIP_BLOCK_SIZE : dirent->crestlen;
 
-	gsf_input_seek (zip->input, (off_t) (dirent->data_offset + dirent->stream.total_in), GSF_SEEK_SET);
+	gsf_input_seek (zip->input, (gsf_off_t) (dirent->data_offset + dirent->stream.total_in), G_SEEK_SET);
 	if ((data = gsf_input_read (zip->input, read_now, NULL)) == NULL)
 		return FALSE;
 
@@ -365,11 +385,6 @@ gsf_infile_zip_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 	GsfInfileZip *zip = GSF_INFILE_ZIP (input);
 	ZipDirent    *dirent = zip->dirent;
 
-	static guint8 *buf;
-
-	if (buf == NULL)
-		buf = g_malloc (256);
-
 	if (dirent->restlen < num_bytes)
 		return NULL;
 
@@ -380,8 +395,15 @@ gsf_infile_zip_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 		break;
 	case ZIP_DEFLATED:
 
-		if (buffer == NULL)
-			buffer = buf;
+		if (buffer == NULL) {
+			if (zip->buf_size < num_bytes) {
+				zip->buf_size = MAX (num_bytes, 256);
+				if (zip->buf != NULL)
+					g_free (zip->buf);
+				zip->buf = g_malloc (zip->buf_size);
+			}
+			buffer = zip->buf;
+		}
 
 		dirent->stream.avail_out = num_bytes;
 		dirent->stream.next_out = (unsigned char *)buffer;
@@ -417,7 +439,7 @@ gsf_infile_zip_read (GsfInput *input, size_t num_bytes, guint8 *buffer)
 }
 
 static gboolean
-gsf_infile_zip_seek (GsfInput *input, off_t offset, GsfOff_t whence)
+gsf_infile_zip_seek (GsfInput *input, gsf_off_t offset, GSeekType whence)
 {
 	GsfInfileZip *zip = GSF_INFILE_ZIP (input);
 
@@ -444,7 +466,7 @@ gsf_infile_zip_new_child (GsfInfileZip *parent, ZipDirent *dirent)
 
 	child = zip_dup (parent);
 
-	gsf_input_set_size (GSF_INPUT (child), dirent->usize);
+	gsf_input_set_size (GSF_INPUT (child), (gsf_off_t) dirent->usize);
 	gsf_input_set_name (GSF_INPUT (child), dirent->name);
 	gsf_input_set_container (GSF_INPUT (child), GSF_INFILE (parent));
 
@@ -454,7 +476,8 @@ gsf_infile_zip_new_child (GsfInfileZip *parent, ZipDirent *dirent)
 	 * should test tons of other info, but trust that those are correct
 	 **/
 
-	if (gsf_input_seek (child->input, dirent->offset, GSF_SEEK_SET) ||
+	if (gsf_input_seek (child->input, (gsf_off_t) dirent->offset,
+			    G_SEEK_SET) ||
 	    NULL == (data = gsf_input_read (child->input, ZIP_FILE_HEADER_SIZE, NULL)) ||
 	    0 != memcmp (data, header_signature, sizeof (header_signature))) {
 		g_object_unref (child);
@@ -550,6 +573,8 @@ gsf_infile_zip_finalize (GObject *obj)
 
 	g_list_free (zip->dirent_list);
 
+	g_free (zip->buf);
+
 	parent_class = g_type_class_peek (GSF_INFILE_TYPE);
 	if (parent_class && parent_class->finalize)
 		parent_class->finalize (obj);
@@ -604,12 +629,14 @@ gsf_infile_zip_new (GsfInput *source, GError **err)
 	zip = g_object_new (GSF_INFILE_ZIP_TYPE, NULL);
 	g_object_ref (G_OBJECT (source));
 	zip->input = source;
-	gsf_input_set_size (GSF_INPUT (zip), 0);
+	gsf_input_set_size (GSF_INPUT (zip), (gsf_off_t) 0);
 
 	if (zip_init_info (zip, err)) {
 		g_object_unref (G_OBJECT (zip));
 		return NULL;
 	}
+	zip->buf		= NULL;
+	zip->buf_size		= 0;
 
 	return GSF_INFILE (zip);
 }
