@@ -108,14 +108,15 @@ gsf_outfile_msole_finalize (GObject *obj)
 }
 
 static gboolean
-gsf_outfile_msole_seek (GsfOutput *output, off_t offset, GsfOff_t whence)
+gsf_outfile_msole_seek (GsfOutput *output, gsf_off_t offset,
+			GSeekType whence)
 {
 	GsfOutfileMSOle *ole = (GsfOutfileMSOle *)output;
 
 	switch (whence) {
-	case GSF_SEEK_SET : break;
-	case GSF_SEEK_CUR : offset += output->cur_offset;	break;
-	case GSF_SEEK_END : offset += output->cur_size;		break;
+	case G_SEEK_SET : break;
+	case G_SEEK_CUR : offset += output->cur_offset;	break;
+	case G_SEEK_END : offset += output->cur_size;		break;
 	default : return TRUE;
 	}
 
@@ -135,8 +136,8 @@ gsf_outfile_msole_seek (GsfOutput *output, off_t offset, GsfOff_t whence)
 
 	case MSOLE_BIG_BLOCK:
 		return gsf_output_seek (ole->sink,
-			(off_t)(ole->content.big_block.start_offset + offset),
-			GSF_SEEK_SET);
+			(gsf_off_t)(ole->content.big_block.start_offset + offset),
+			G_SEEK_SET);
 
 	default :
 		return TRUE;
@@ -148,7 +149,7 @@ gsf_outfile_msole_seek (GsfOutput *output, off_t offset, GsfOff_t whence)
 static void
 bb_pad_zero (GsfOutput *out)
 {
-	unsigned size = 1 << OLE_DEFAULT_BB_SHIFT;
+	unsigned size = OLE_DEFAULT_BB_SIZE;
 	static guint8 const zeros [] = {
 		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -190,6 +191,7 @@ ole_write_bat (GsfOutput *sink, guint32 block, unsigned blocks)
 {
 	guint8 buf [BAT_INDEX_SIZE];
 
+#warning optimize this to dump a buffer in 1 step
 	while (blocks-- > 1) {
 		block++;
 		GSF_LE_SET_GUINT32 (buf, block);
@@ -199,6 +201,24 @@ ole_write_bat (GsfOutput *sink, guint32 block, unsigned blocks)
 	gsf_output_write (sink, BAT_INDEX_SIZE, buf);
 }
 
+static void
+ole_write_const (GsfOutput *sink, guint32 value, unsigned n)
+{
+	guint8 buf [BAT_INDEX_SIZE];
+
+	GSF_LE_SET_GUINT32 (buf, value);
+	while (n-- > 0)
+		gsf_output_write (sink, BAT_INDEX_SIZE, buf);
+}
+
+static void
+ole_pad_bat_unused (GsfOutput *sink)
+{
+	ole_write_const (sink, BAT_MAGIC_UNUSED, 
+		(unsigned) (OLE_DEFAULT_BB_SIZE -
+			    ((sink->cur_size - OLE_HEADER_SIZE) % OLE_DEFAULT_BB_SIZE) / BAT_INDEX_SIZE));
+}
+
 /**
  * ole_cur_block :
  * @ole :
@@ -221,7 +241,8 @@ gsf_outfile_msole_close (GsfOutput *output)
 	guint8  buf [OLE_HEADER_SIZE];
 	guint32	sbat_start, num_sbat, sb_data_start, sb_data_size, sb_data_blocks;
 	guint32	bat_start, num_bat, dirent_start, num_dirent_blocks, next, child_index;
-	unsigned i, j, blocks, xbats, xbat_pos;
+	unsigned i, j, blocks, num_xbat, xbat_pos;
+	gsf_off_t data_size;
 
 	/* The root dir */
 	if (gsf_output_container (output) == NULL) {
@@ -230,15 +251,13 @@ gsf_outfile_msole_close (GsfOutput *output)
 		/* write small block data */
 		blocks = 0;
 		sb_data_start = ole_cur_block (ole);
-		sb_data_size = gsf_output_tell (ole->sink);
+		data_size = gsf_output_tell (ole->sink);
 		for (i = 0 ; i < elem->len ; i++) {
 			GsfOutfileMSOle *child = g_ptr_array_index (elem, i);
 			if (child->type == MSOLE_SMALL_BLOCK) {
-				size_t size = gsf_output_size (GSF_OUTPUT (child));
+				gsf_off_t size = gsf_output_size (GSF_OUTPUT (child));
 
-				child->blocks = size >> OLE_DEFAULT_SB_SHIFT;
-				if ((child->blocks << OLE_DEFAULT_SB_SHIFT) != size)
-					child->blocks++;
+				child->blocks = ((size - 1) >> OLE_DEFAULT_SB_SHIFT) + 1;
 				gsf_output_write (ole->sink, child->blocks << OLE_DEFAULT_SB_SHIFT,
 					child->content.small_block.buf);
 
@@ -246,7 +265,13 @@ gsf_outfile_msole_close (GsfOutput *output)
 				blocks += child->blocks;
 			}
 		}
-		sb_data_size = gsf_output_tell (ole->sink) - sb_data_size;
+		data_size = gsf_output_tell (ole->sink) - data_size;
+		sb_data_size = data_size;
+		if ((gsf_off_t) sb_data_size != data_size) {
+			/* Check for overflow */
+			g_warning ("File too big");
+			return FALSE;
+		}
 		bb_pad_zero (ole->sink);
 		sb_data_blocks = ole_cur_block (ole) - sb_data_start;
 
@@ -257,7 +282,7 @@ gsf_outfile_msole_close (GsfOutput *output)
 			if (child->type == MSOLE_SMALL_BLOCK)
 				ole_write_bat (ole->sink,  child->first_block, child->blocks);
 		}
-		bb_pad_zero (ole->sink);
+		ole_pad_bat_unused (ole->sink);
 		num_sbat = ole_cur_block (ole) - sbat_start;
 
 		/* write dirents */
@@ -281,7 +306,8 @@ gsf_outfile_msole_close (GsfOutput *output)
 
 			if (child->root == child) {
 				GSF_LE_SET_GUINT8  (buf + DIRENT_TYPE,	DIRENT_TYPE_ROOTDIR);
-				GSF_LE_SET_GUINT32 (buf + DIRENT_FIRSTBLOCK, sb_data_start);
+				GSF_LE_SET_GUINT32 (buf + DIRENT_FIRSTBLOCK,
+					(sb_data_size > 0) ? sb_data_start : BAT_MAGIC_END_OF_CHAIN);
 				GSF_LE_SET_GUINT32 (buf + DIRENT_FILE_SIZE, sb_data_size);
 			} else if (child->type == MSOLE_DIR) {
 				GSF_LE_SET_GUINT8 (buf + DIRENT_TYPE, DIRENT_TYPE_DIR);
@@ -331,38 +357,63 @@ gsf_outfile_msole_close (GsfOutput *output)
 			if (child->type == MSOLE_BIG_BLOCK)
 				ole_write_bat (ole->sink, child->first_block, child->blocks);
 		}
-		ole_write_bat (ole->sink, sb_data_start, sb_data_blocks);
-		ole_write_bat (ole->sink, sbat_start, num_sbat);
+		if (sb_data_blocks > 0)
+			ole_write_bat (ole->sink, sb_data_start, sb_data_blocks);
+		if (num_sbat > 0)
+			ole_write_bat (ole->sink, sbat_start, num_sbat);
 		ole_write_bat (ole->sink, dirent_start, num_dirent_blocks);
-		bb_pad_zero (ole->sink);
-		num_bat = ole_cur_block (ole) - bat_start;
 
-		/* fix up the header */
-		GSF_LE_SET_GUINT32 (buf,   num_bat);
-		GSF_LE_SET_GUINT32 (buf+4, dirent_start);
-		gsf_output_seek (ole->sink, OLE_HEADER_NUM_BAT, GSF_SEEK_SET);
-		gsf_output_write (ole->sink, 8, buf);
+		/* List the BAT and meta-BAT blocks in the BAT.  Doing this may
+		 * increase the size of the bat and hence the metabat, so be
+		 * prepared to iterate.
+		 */
+		num_bat = 0;
+		num_xbat = 0;
+recalc_bat_bat :
+		i = ((ole->sink->cur_size
+		      + BAT_INDEX_SIZE * (num_bat + num_xbat)
+		      - OLE_HEADER_SIZE - 1) >> OLE_DEFAULT_BB_SHIFT) + 1;
+		i -= bat_start;
+		if (num_bat != i) {
+			num_bat = i;
+			goto recalc_bat_bat;
+		}
+		i = 0;
+		if (num_bat > OLE_HEADER_METABAT_SIZE)
+			i = (num_bat - OLE_HEADER_METABAT_SIZE - 1)
+				/ OLE_DEFAULT_METABAT_SIZE;
+		if (num_xbat != i) {
+			num_xbat = i;
+			goto recalc_bat_bat;
+		}
 
-		GSF_LE_SET_GUINT32 (buf+0x0, sbat_start);
-		GSF_LE_SET_GUINT32 (buf+0x4, num_sbat);
-		if (num_bat > OLE_HEADER_METABAT_SIZE) {
-			unsigned rem = num_bat - OLE_HEADER_METABAT_SIZE;
-
-			xbats = rem / OLE_DEFAULT_METABAT_SIZE;
-			if ((xbats * OLE_DEFAULT_METABAT_SIZE) != rem)
-				xbats++;
+		ole_write_const (ole->sink, BAT_MAGIC_BAT, num_bat);
+		ole_write_const (ole->sink, BAT_MAGIC_METABAT, num_xbat);
+		ole_pad_bat_unused (ole->sink);
 
+		if (num_xbat > 0) {
 			/* 1st xbat is immediately after bat */
 			xbat_pos = bat_start + num_bat;
 			blocks = OLE_HEADER_METABAT_SIZE;
 		} else {
 			xbat_pos = BAT_MAGIC_END_OF_CHAIN;
 			blocks = num_bat;
-			xbats = 0;
 		}
+
+		/* fix up the header */
+		GSF_LE_SET_GUINT32 (buf,   num_bat);
+		GSF_LE_SET_GUINT32 (buf+4, dirent_start);
+		gsf_output_seek (ole->sink,
+			(gsf_off_t) OLE_HEADER_NUM_BAT, G_SEEK_SET);
+		gsf_output_write (ole->sink, 8, buf);
+
+		GSF_LE_SET_GUINT32 (buf+0x0,
+			(num_sbat > 0) ? sbat_start : BAT_MAGIC_END_OF_CHAIN);
+		GSF_LE_SET_GUINT32 (buf+0x4, num_sbat);
 		GSF_LE_SET_GUINT32 (buf+0x8, xbat_pos);
-		GSF_LE_SET_GUINT32 (buf+0xc, xbats);
-		gsf_output_seek (ole->sink, OLE_HEADER_SBAT_START, GSF_SEEK_SET);
+		GSF_LE_SET_GUINT32 (buf+0xc, num_xbat);
+		gsf_output_seek (ole->sink, (gsf_off_t) OLE_HEADER_SBAT_START,
+				 G_SEEK_SET);
 		gsf_output_write (ole->sink, 0x10, buf);
 
 		/* write initial Meta-BAT */
@@ -374,8 +425,9 @@ gsf_outfile_msole_close (GsfOutput *output)
 		/* write extended Meta-BAT */
 		if (blocks < num_bat) {
 			/* Append the meta bats */
-			gsf_output_seek (ole->sink, 0, GSF_SEEK_END);
-			for (i = 0 ; i++ < xbats ; ) {
+			gsf_output_seek (ole->sink,
+					 (gsf_off_t) 0, G_SEEK_END);
+			for (i = 0 ; i++ < num_xbat ; ) {
 				bat_start += blocks;
 				blocks = num_bat - bat_start;
 				if (blocks > OLE_DEFAULT_METABAT_SIZE)
@@ -385,7 +437,7 @@ gsf_outfile_msole_close (GsfOutput *output)
 					gsf_output_write (ole->sink, BAT_INDEX_SIZE, buf);
 				}
 
-				if (xbats != 0) {
+				if (num_xbat != 0) {
 					xbat_pos++;
 					GSF_LE_SET_GUINT32 (buf, xbat_pos);
 					gsf_output_write (ole->sink, BAT_INDEX_SIZE, buf);
@@ -402,7 +454,7 @@ gsf_outfile_msole_close (GsfOutput *output)
 
 		return gsf_output_close (ole->sink);
 	} else if (ole->type == MSOLE_BIG_BLOCK) {
-		gsf_outfile_msole_seek (output, 0, GSF_SEEK_END);
+		gsf_outfile_msole_seek (output, (gsf_off_t) 0, G_SEEK_END);
 		bb_pad_zero (ole->sink);
 		ole->blocks = ole_cur_block (ole) - ole->first_block;
 		return gsf_output_unwrap (ole->sink, output);
@@ -415,11 +467,13 @@ gsf_outfile_msole_write (GsfOutput *output,
 			 size_t num_bytes, guint8 const *data)
 {
 	GsfOutfileMSOle *ole = (GsfOutfileMSOle *)output;
+	size_t wsize;
 
 	g_return_val_if_fail (ole->type != MSOLE_DIR, FALSE);
 	if (ole->type == MSOLE_SMALL_BLOCK) {
 		gboolean ok;
 		guint8 *buf;
+		gsf_off_t start_offset;
 
 		if ((output->cur_offset + num_bytes) < OLE_DEFAULT_THRESHOLD) {
 			memcpy (ole->content.small_block.buf + output->cur_offset,
@@ -432,10 +486,24 @@ gsf_outfile_msole_write (GsfOutput *output,
 
 		buf = ole->content.small_block.buf;
 		ole->content.small_block.buf = NULL;
-		ole->content.big_block.start_offset = gsf_output_tell (ole->sink);
+		start_offset = gsf_output_tell (ole->sink);
+		ole->content.big_block.start_offset = start_offset;
+		if ((gsf_off_t) ole->content.big_block.start_offset
+		    != start_offset) {
+			/* Check for overflow */
+			g_warning ("File too big");
+			return FALSE;
+		}
+		
 		ole->first_block = ole_cur_block (ole);
 		ole->type = MSOLE_BIG_BLOCK;
-		gsf_output_write (ole->sink, output->cur_size, buf);
+		wsize = output->cur_size;
+		if ((gsf_off_t) wsize != output->cur_size) {
+			/* Check for overflow */
+			g_warning ("File too big");
+			return FALSE;
+		}
+		gsf_output_write (ole->sink, wsize, buf);
 		g_free (buf);
 	}
 
