@@ -33,11 +33,14 @@ typedef struct _GsfInfile 	GsfInfile;
 typedef struct _GsfOutput	GsfOutput;
 typedef struct _GsfOutfile 	GsfOutfile;
 
-typedef enum {
-	GSF_SEEK_SET,
-	GSF_SEEK_CUR,
-	GSF_SEEK_END
-} GsfOff_t;
+typedef struct _GsfDocMetaData	GsfDocMetaData;
+typedef struct _GsfTimestamp	GsfTimestamp;
+
+/* FIXME:
+ * gsf_off_t is really supposed to be the widest type off_t can be configured
+ * to on the platform
+ */ 
+typedef gint64 gsf_off_t;
 
 G_END_DECLS
 
